@@ -3,7 +3,16 @@ import pytest
 from dvc_data.fs import DataFileSystem
 from dvc_data.hashfile.db import HashFileDB
 from dvc_data.hashfile.hash_info import HashInfo
-from dvc_data.index import DataIndex, DataIndexEntry, build, checkout, commit
+from dvc_data.hashfile.meta import Meta
+from dvc_data.index import (
+    DataIndex,
+    DataIndexEntry,
+    build,
+    checkout,
+    collect,
+    commit,
+    save,
+)
 
 
 @pytest.fixture
@@ -72,6 +81,55 @@ def test_fs(tmp_upath, odb, as_filesystem):
     ]
 
 
+def test_collect(tmp_upath, odb, as_filesystem):
+    (tmp_upath / "foo").write_bytes(b"foo\n")
+    (tmp_upath / "data").mkdir()
+    (tmp_upath / "data" / "bar").write_bytes(b"bar\n")
+    (tmp_upath / "data" / "baz").write_bytes(b"baz\n")
+
+    index = DataIndex(
+        {
+            ("foo",): DataIndexEntry(odb=odb, cache=odb),
+            ("data",): DataIndexEntry(odb=odb, cache=odb),
+        },
+    )
+    fs = as_filesystem(tmp_upath.fs)
+    collect(index, tmp_upath, fs)
+    assert index[("foo",)].meta == Meta(size=4)
+    assert index[("foo",)].fs == fs
+    assert index[("foo",)].path == str(tmp_upath / "foo")
+    assert index[("data",)].meta.isdir
+    assert index[("data",)].fs == fs
+    assert index[("data",)].path == str(tmp_upath / "data")
+    assert index[("data", "bar")].meta == Meta(size=4)
+    assert index[("data", "bar")].fs == fs
+    assert index[("data", "bar")].path == str(tmp_upath / "data" / "bar")
+    assert index[("data", "baz")].meta == Meta(size=4)
+    assert index[("data", "baz")].fs == fs
+    assert index[("data", "baz")].path == str(tmp_upath / "data" / "baz")
+
+
+def test_save(tmp_upath, odb, as_filesystem):
+    (tmp_upath / "foo").write_bytes(b"foo\n")
+    (tmp_upath / "data").mkdir()
+    (tmp_upath / "data" / "bar").write_bytes(b"bar\n")
+    (tmp_upath / "data" / "baz").write_bytes(b"baz\n")
+
+    index = DataIndex(
+        {
+            ("foo",): DataIndexEntry(odb=odb, cache=odb),
+            ("data",): DataIndexEntry(odb=odb, cache=odb),
+        },
+    )
+    fs = as_filesystem(tmp_upath.fs)
+    collect(index, tmp_upath, fs)
+    save(index)
+    assert odb.exists("d3b07384d113edec49eaa6238ad5ff00")
+    assert odb.exists("1f69c66028c35037e8bf67e5bc4ceb6a.dir")
+    assert odb.exists("c157a79031e1c40f85931829bc5fc552")
+    assert odb.exists("258622b1688250cb619f3c9ccaefb7eb")
+
+
 def test_build(tmp_upath, odb, as_filesystem):
     (tmp_upath / "foo").write_text("foo\n")
     (tmp_upath / "data").mkdir()
